@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 
 #
-# Copyright (c) 2016 Red Hat, Inc.
+# Copyright oVirt Authors
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -21,7 +21,6 @@ import json
 import os
 import pycurl
 import re
-import six
 import sys
 import threading
 
@@ -31,10 +30,14 @@ except ImportError:
     from urllib import urlencode
     from urlparse import urlparse
 
-from ovirtsdk4 import version
 from ovirtsdk4.http import Response
 
 
+def get_version():
+    from ovirtsdk4 import version
+    return version.VERSION
+
+
 class Error(Exception):
     """
     General exception which is thrown by SDK,
@@ -44,20 +47,24 @@ class Error(Exception):
     def __init__(self, message, code=None, fault=None):
         """
         Creates an instance of Error class.
+        """
+        super(Error, self).__init__(message)
+        """
+        Creates an instance of Error class.
 
-        `message`:: The exception message.
-
-        `code`:: An error code associated to the error. For HTTP related
+        :param message: The exception message.
+        """
+        self.code = code
+        """
+        An error code associated to the error. For HTTP related
         errors, this will be the HTTP response code returned by the server.
         For example, if retrieving of a virtual machine fails because it
         doesn't exist this attribute will contain the integer value 404. Note
-        that this may be `nil` if the error is not HTTP related.
-
-        `fault`:: The `Fault` object associated to the error.
-        """
-        super(Error, self).__init__(message)
-        self.code = code
+        that this may be `nil` if the error is not HTTP related."""
         self.fault = fault
+        """
+        The `Fault` object associated to the error.
+        """
 
 
 class AuthError(Error):
@@ -181,7 +188,8 @@ class Connection(object):
     __XML_CONTENT_TYPE_RE = re.compile(r"^\s*(application|text)/xml\s*(;.*)?$")
 
     # Regular expression used to check JSON content type.
-    __JSON_CONTENT_TYPE_RE = re.compile(r"^\s*(application|text)/json\s*(;.*)?$")
+    __JSON_CONTENT_TYPE_RE = \
+        re.compile(r"^\s*(application|text)/json\s*(;.*)?$")
 
     # The typical URL path, used just to generate informative error messages.
     __TYPICAL_PATH = '/ovirt-engine/api'
@@ -230,72 +238,74 @@ class Connection(object):
 
         This method supports the following parameters:
 
-        `url`:: A string containing the base URL of the server, usually
+        :param url: A string containing the base URL of the server, usually
         something like `https://server.example.com/ovirt-engine/api`.
 
-        `username`:: The name of the user, something like `admin@internal`.
+        :param username: The name of the user, something like `admin@internal`.
 
-        `password`:: The name password of the user.
+        :param password: The name password of the user.
 
-        `token`:: : The token to be used to access API. Optionally, user can
+        :param token: The token to be used to access API.
+        The token to be used to access API. Optionally, user can
         use token, instead of username and password to access API. If user
         don't specify `token` parameter, SDK will automatically create one.
 
-        `insecure`:: A boolean flag that indicates if the server TLS
+        :param insecure: A boolean flag that indicates if the server TLS
         certificate and host name should be checked.
 
-        `ca_file`:: A PEM file containing the trusted CA certificates. The
+        :param ca_file: A PEM file containing the trusted CA certificates. The
         certificate presented by the server will be verified using these CA
         certificates. If `ca_file` parameter is not set, system wide
         CA certificate store is used.
 
-        `debug`:: A boolean flag indicating if debug output should be
+        :param debug: A boolean flag indicating if debug output should be
+        A boolean flag indicating if debug output should be
         generated. If the value is `True` and the `log` parameter isn't
         `None` then the data sent to and received from the server will
         be written to the log. Be aware that user names and passwords will
         also be written, so handle it with care.
 
-        `log`:: The logger where the log messages will be written.
+        :param log: The logger where the log messages will be written.
 
-        `kerberos`:: A boolean flag indicating if Kerberos
+        :param kerberos: A boolean flag indicating if Kerberos
         authentication should be used instead of the default basic
         authentication.
 
-        `timeout`:: The maximum total time to wait for the response, in
+        :param timeout: The maximum total time to wait for the response, in
         seconds. A value of zero (the default) means wait for ever. If
         the timeout expires before the response is received an exception
         will be raised.
 
-        `compress`:: A boolean flag indicating if the SDK should ask
+        :param compress: A boolean flag indicating if the SDK should ask
         the server to send compressed responses. The default is `True`.
         Note that this is a hint for the server, and that it may return
         uncompressed data even when this parameter is set to `True`.
         Note that compression will be disabled if user pass `debug`
         parameter set to `true`, so the debug messages are in plain text.
 
-        `sso_url`:: A string containing the base SSO URL of the serve.
+        :param sso_url: A string containing the base SSO URL of the serve.
         Default SSO url is computed from the `url` if no `sso_url` is provided.
 
-        `sso_revoke_url`:: A string containing the base URL of the SSO
+        :param sso_revoke_url: A string containing the base URL of the SSO
         revoke service. This needs to be specified only when using
         an external authentication service. By default this URL
         is automatically calculated from the value of the `url` parameter,
         so that SSO token revoke will be performed using the SSO service
         that is part of the engine.
 
-        `sso_token_name`:: The token name in the JSON SSO response returned
+        :param sso_token_name: The token name in the JSON SSO response returned
         from the SSO server. Default value is `access_token`.
 
-        `headers`:: A dictionary with headers which should be send with every
-        request.
+        :param headers: A dictionary with headers which should be send with
+        every request.
 
-        `connections`:: The maximum number of connections to open to the host.
-        If the value is `0` (the default) then the number of connections will
-        be unlimited.
+        :param connections: The maximum number of connections to open to the
+        host. If the value is `0` (the default) then the number of connections
+        will be unlimited.
 
-        `pipeline`:: The maximum number of request to put in an HTTP pipeline
-        without waiting for the response. If the value is `0` (the default)
-        then pipelining is disabled.
+        :param pipeline: The maximum number of request to put in an HTTP
+        pipeline without waiting for the response. If the value is `0`
+        (the default) then pipelining is disabled.
         """
 
         # Check mandatory parameters:
@@ -360,7 +370,7 @@ class Connection(object):
 
         This method supports the following parameters.
 
-        `request`:: The Request object containing the details of the HTTP
+        :param request: The Request object containing the details of the HTTP
         request to send.
 
         The returned value is a Request object containing the details of the
@@ -372,10 +382,10 @@ class Connection(object):
 
     def authenticate(self):
         """
-        Return token which can be used for authentication instead of credentials.
-        It will be created, if it not exists, yet. By default the token will be
-        revoked when the connection is closed, unless the `logout` parameter of
-        the `close` method is `False`.
+        Return token which can be used for authentication instead of
+        credentials. It will be created, if it not exists, yet.
+        By default the token will be revoked when the connection is closed,
+        unless the `logout` parameter of the `close` method is `False`.
         """
         try:
             if self._sso_token is None:
@@ -445,7 +455,7 @@ class Connection(object):
         for header_name, header_value in headers_dict.items():
             header_lines.append('%s: %s' % (header_name, header_value))
 
-        header_lines.append('User-Agent: PythonSDK/%s' % version.VERSION)
+        header_lines.append('User-Agent: PythonSDK/%s' % get_version())
         header_lines.append('Version: 4')
         header_lines.append('Content-Type: application/xml')
         header_lines.append('Accept: application/xml')
@@ -519,9 +529,9 @@ class Connection(object):
                     # If the request failed because of authentication, and it
                     # wasn't a request to the SSO service, then the most likely
                     # cause is an expired SSO token. In this case we need to
-                    # request a new token, and try the original request again, but
-                    # only once. It if fails again, we just return the failed
-                    # response.
+                    # request a new token, and try the original request again,
+                    # but only once. If it fails again, we just return the
+                    # failed response.
                     if response.code == 401 and not failed_auth:
                         self._sso_token = self._get_access_token()
                         context = self.__send(context[3])
@@ -633,9 +643,11 @@ class Connection(object):
     def _get_sso_error(self, sso_response):
         # OpenId define `error_description` field, OAuth doesn't:
         if 'error_description' in sso_response:
-            sso_error = (sso_response.get('error'), sso_response.get('error_description'))
+            sso_error = (sso_response.get('error'),
+                         sso_response.get('error_description'))
         else:
-            sso_error = (sso_response.get('error_code'), sso_response.get('error'))
+            sso_error = (sso_response.get('error_code'),
+                         sso_response.get('error'))
 
         return sso_error
 
@@ -668,7 +680,7 @@ class Connection(object):
 
         # Prepare headers:
         header_lines = [
-            'User-Agent: PythonSDK/%s' % version.VERSION,
+            'User-Agent: PythonSDK/%s' % get_version(),
             'Accept: application/json'
         ]
         curl.setopt(pycurl.HTTPHEADER, header_lines)
@@ -699,7 +711,8 @@ class Connection(object):
         header_lines = headers_text.split('\n')
 
         # Check the returned content type:
-        self._check_content_type(self.__JSON_CONTENT_TYPE_RE, 'JSON', header_lines)
+        self._check_content_type(self.__JSON_CONTENT_TYPE_RE, 'JSON',
+                                 header_lines)
 
         return json.loads(body_buf.getvalue().decode('utf-8'))
 
@@ -742,7 +755,7 @@ class Connection(object):
             return True
         except Error:
             if raise_exception:
-                six.reraise(*sys.exc_info())
+                raise
             return False
         except Exception as exception:
             if raise_exception:
@@ -784,7 +797,8 @@ class Connection(object):
             )
 
         # Remove the prefix from the URL, follow the path to the relevant
-        # service and invoke the "get", or "list method to retrieve its representation:
+        # service and invoke the "get", or "list method to retrieve its
+        # representation:
         path = href[len(prefix):]
         service = self.service(path)
         if isinstance(obj, List):
@@ -796,7 +810,7 @@ class Connection(object):
         """
         Releases the resources used by this connection.
 
-        `logout`:: A boolean, which specify if token should be revoked,
+        `logout` A boolean, which specify if token should be revoked,
         and so user should be logged out.
         """
 
@@ -818,10 +832,10 @@ class Connection(object):
 
         This method supports the following parameters:
 
-        `path`:: The path that will be added to the base URL. The default is an
+        `path` The path that will be added to the base URL. The default is an
         empty string.
 
-        `query`:: A dictionary containing the query parameters to add to the
+        `query` A dictionary containing the query parameters to add to the
         URL. The keys of the dictionary should be strings containing the names
         of the parameters, and the values should be strings containing the
         values.
@@ -841,7 +855,7 @@ class Connection(object):
          XML then it does nothing. If it isn't XML then it raises an
          exception.
 
-         `response`:: The HTTP response to check.
+         `response` The HTTP response to check.
         """
         return self._check_content_type(
             self.__XML_CONTENT_TYPE_RE,
@@ -855,7 +869,7 @@ class Connection(object):
          JSON then it does nothing. If it isn't JSON then it raises an
          exception.
 
-         `response`:: The HTTP response to check.
+         `response` The HTTP response to check.
         """
         return self._check_content_type(
             self.__JSON_CONTENT_TYPE_RE,
@@ -868,14 +882,15 @@ class Connection(object):
         Checks the given content type and raises an exception if it isn't the
         expected one.
 
-        `expected_re`:: The regular expression used to check the expected
+        `expected_re` The regular expression used to check the expected
                         content type.
-        `expected_name`:: The name of the expected content type.
-        `headers`:: The HTTP headers to check.
+        `expected_name` The name of the expected content type.
+        `headers` The HTTP headers to check.
         """
         content_type = self._get_header_value(headers, 'content-type')
         if expected_re.match(content_type) is None:
-            msg = "The response content type '{}' isn't the expected {}".format(
+            msg = "The response content type '{}' isn't the expected {}"
+            msg = msg.format(
                 content_type,
                 expected_name,
             )
@@ -892,7 +907,7 @@ class Connection(object):
         """
         Read the response.
 
-        `context`:: tuple which contains cur easy, response body,
+        `context` tuple which contains cur easy, response body,
         response headers, original request
         """
         # Extract the response code and body:
@@ -929,17 +944,18 @@ class Connection(object):
         elif e_code == pycurl.E_OPERATION_TIMEOUTED:
             clazz = TimeoutError
 
-        six.reraise(clazz, clazz(error_msg), sys.exc_info()[2])
+        raise clazz(error_msg).with_traceback(sys.exc_info()[2])
 
     def _get_header_value(self, headers, name):
         """
         Return header value by its name.
 
-        `headers`:: list of headers
-        `name`:: name of the header
+        :param headers: list of headers
+        :param name: name of the header
         """
         return next(
-            (h.split(':')[1].strip() for h in headers if h.lower().startswith(name)),
+            (h.split(':')[1].strip() for h in headers
+             if h.lower().startswith(name)),
             None
         )
 
@@ -958,7 +974,7 @@ class Connection(object):
         # some as arrays of bytes, so we need to check the type of the
         # provided data and convert it to strings before trying to
         # manipulate it with the "replace", "strip" and "split" methods:
-        text = data.decode('utf-8') if type(data) == bytes else data
+        text = data.decode('utf-8') if isinstance(data, bytes) else data
 
         # Split the debug data into lines and send a debug message for
         # each line:
@@ -984,8 +1000,7 @@ class ConnectionBuilder(object):
     equivalent to calling the constructor of the `Connection`
     class. Typical use will be like this:
 
-    [source,python]
-    ----
+    ```python
     # Create the builder once:
     builder = ConnectionBuilder(
         url='https://enginer40.example.com/ovirt-engine/api',
@@ -1001,7 +1016,7 @@ class ConnectionBuilder(object):
     # Create and use a second connection:
     with builder.build() as connection:
        ...
-    ----
+    ```
     """
 
     def __init__(self, **kwargs):
@@ -1029,7 +1044,7 @@ class ConnectionBuilder(object):
 #   import ovirtsdk4 as sdk
 #   vm = sdk.types.Vm()
 #
-from ovirtsdk4 import readers  # noqa: F401
-from ovirtsdk4 import services  # noqa: F401
-from ovirtsdk4 import types  # noqa: F401
-from ovirtsdk4 import writers  # noqa: F401
+import ovirtsdk4.readers as readers  # noqa: E402, F401
+import ovirtsdk4.writers as writers  # noqa: E402, F401
+import ovirtsdk4.types as types  # noqa: E402, F401
+import ovirtsdk4.services as services  # noqa: E402, F401
